<?php

namespace Modules\CustomFields\Traits;

use Illuminate\Support\Str;
use Modules\CustomFields\Models\Field;
use Modules\CustomFields\Models\Location;

trait CustomFields
{
    public function getFieldsByLocation($code)
    {
        $location = Location::code($code)->first();

        if (is_null($location)) {
            return collect([]);
        }

        return Field::enabled()->orderBy('name')->byLocation($location->id)->get();
    }

    public function getInputAttributes(Field $field): array
    {
        switch ($field->type->type) {
            case 'date':
            case 'time':
            case 'dateTime':
            case 'text':
            case 'textarea':
                $attributes = [
                    'v-model' => 'form.' . $field->code,
                ];

                break;

            case 'select':
                $attributes = [
                    'v-model' => 'form.' . $field->code,
                    'model' => 'this.' . $field->code,
                ];

                break;

            default:
                $attributes = [];

                break;
        }

        if ($field->required) {
            $attributes['required'] = 'required';
        }

        if ($field->type->type == 'textarea') {
            $attributes['rows'] = '3';
        }

        return $attributes;
    }

    public function getInputAttributesForItems(Field $field): array
    {
        switch ($field->type->type) {
            case 'text':
            case 'textarea':
                $attributes = [
                    'data-item' => $field->code,
                    'v-model' => null,
                    ':value' => 'row.' . $field->code . ' = this.' . $field->code . '[index]',
                    '@input' => 'row.' . $field->code . ' = $event.target.value; this.' . $field->code . '[index] = $event.target.value; onBindingItemField(index, "' . $field->code . '")',
                ];

                break;

            case 'date':
            case 'time':
            case 'dateTime':
                $attributes = [
                    'data-item' => $field->code,
                    'v-model' => 'row.' . $field->code,
                    'change' => 'onBindingItemField(index, "' . $field->code . '")',
                    'model' => 'this.' . $field->code . '[index]',
                ];

                break;

            case 'select':
                $attributes = [
                    'data-item' => $field->code,
                    'v-model' => 'row.' . $field->code,
                    'visible-change' => 'onBindingItemField(index, "' . $field->code . '")',
                    'model' => 'this.' . $field->code . '[index]',
                ];

                break;

            case 'checkbox':
                $attributes = [
                    ':id' => '"checkbox-' . $field->code . '-:item_id-" + index',
                    'data-item' => $field->code,
                    '@change' => 'onBindingItemField(index, "' . $field->code . '")',
                    'v-model' => 'row.' . $field->code,
                ];

                break;

            default:
                $attributes = [];

                break;
        }

        if ($field->required) {
            $attributes['required'] = 'required';
        }

        if ($field->type->type == 'textarea') {
            $attributes['rows'] = '3';
        }

        return $attributes;
    }

    public function getInputValues($field, $request)
    {
        $model = $request->route(Str::singular((string) $request->segment(2)));

        if (is_null($model)) {
            return null;
        }

        $field_value = $field->field_values()->record($model->id, get_class($model))->first();

        if (is_null($field_value)) {
            return null;
        }

        return $field_value->value;
    }

    public function getInputValuesForItems($field, $request): array
    {
        $values = [];

        if (!$request->routeIs('invoices.edit') && !$request->routeIs('bills.edit')) {
            return $values;
        }

        $document = $request->route(Str::singular((string) $request->segment(2)));

        foreach ($document->items as $item) {
            $field_value = $field->field_values()->record($item->id, get_class($item))->first();

            if (is_null($field_value)) {
                continue;
            }

            $values[] = $field_value->value;
        }

        return $values;
    }

    public function getFieldValue($field, $request, $view)
    {
        $model = $request->route(Str::singular((string) $request->segment(2)));

        if (is_null($model)) {
            return null;
        }

        if ($item_model = $view->getData()['item']) {
            $field_value = $field->field_values()->record($item_model->id, get_class($item_model))->first();
        } else {
            $field_value = $field->field_values()->record($model->id, get_class($model))->first();
        }

        if (is_null($field_value)) {
            return null;
        }

        $value = $field_value->value;

        $field_type_options = $field->fieldTypeOption->pluck('value', 'id');

        if (isset($field_type_options[$field_value->value])) {
            $value = $field_type_options[$field_value->value];
        }

        return $value;
    }
}
